@@ -14,7 +14,6 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import argparse
-import io
 import locale
 import os
 import platform
@@ -47,7 +46,7 @@ def supports_color():
             hasattr(sys.stdout, 'isatty') and sys.stdout.isatty())
 
 
-class Format(object):
+class Format:
     @staticmethod
     def parsable(problem, filename):
         return ('%(file)s:%(line)s:%(column)s: [%(level)s] %(message)s' %
@@ -142,6 +141,19 @@ def show_problems(problems, file, args_format, no_warn):
     return max_level
 
 
+def find_project_config_filepath(path='.'):
+    for filename in ('.yamllint', '.yamllint.yaml', '.yamllint.yml'):
+        filepath = os.path.join(path, filename)
+        if os.path.isfile(filepath):
+            return filepath
+
+    if os.path.abspath(path) == os.path.abspath(os.path.expanduser('~')):
+        return None
+    if os.path.abspath(path) == os.path.abspath(os.path.join(path, '..')):
+        return None
+    return find_project_config_filepath(path=os.path.join(path, '..'))
+
+
 def run(argv=None):
     parser = argparse.ArgumentParser(prog=APP_NAME,
                                      description=APP_DESCRIPTION)
@@ -158,6 +170,8 @@ def run(argv=None):
     config_group.add_argument('-d', '--config-data', dest='config_data',
                               action='store',
                               help='custom configuration (as YAML source)')
+    parser.add_argument('--list-files', action='store_true', dest='list_files',
+                        help='list files to lint and exit')
     parser.add_argument('-f', '--format',
                         choices=('parsable', 'standard', 'colored', 'github',
                                  'auto'),
@@ -184,6 +198,7 @@ def run(argv=None):
     else:
         user_global_config = os.path.expanduser('~/.config/yamllint/config')
 
+    project_config_filepath = find_project_config_filepath()
     try:
         if args.config_data is not None:
             if args.config_data != '' and ':' not in args.config_data:
@@ -191,12 +206,8 @@ def run(argv=None):
             conf = YamlLintConfig(content=args.config_data)
         elif args.config_file is not None:
             conf = YamlLintConfig(file=args.config_file)
-        elif os.path.isfile('.yamllint'):
-            conf = YamlLintConfig(file='.yamllint')
-        elif os.path.isfile('.yamllint.yaml'):
-            conf = YamlLintConfig(file='.yamllint.yaml')
-        elif os.path.isfile('.yamllint.yml'):
-            conf = YamlLintConfig(file='.yamllint.yml')
+        elif project_config_filepath:
+            conf = YamlLintConfig(file=project_config_filepath)
         elif os.path.isfile(user_global_config):
             conf = YamlLintConfig(file=user_global_config)
         else:
@@ -208,14 +219,20 @@ def run(argv=None):
     if conf.locale is not None:
         locale.setlocale(locale.LC_ALL, conf.locale)
 
+    if args.list_files:
+        for file in find_files_recursively(args.files, conf):
+            if not conf.is_file_ignored(file):
+                print(file)
+        sys.exit(0)
+
     max_level = 0
 
     for file in find_files_recursively(args.files, conf):
         filepath = file[2:] if file.startswith('./') else file
         try:
-            with io.open(file, newline='') as f:
+            with open(file, newline='') as f:
                 problems = linter.run(f, conf, filepath)
-        except EnvironmentError as e:
+        except OSError as e:
             print(e, file=sys.stderr)
             sys.exit(-1)
         prob_level = show_problems(problems, file, args_format=args.format,
@@ -226,7 +243,7 @@ def run(argv=None):
     if args.stdin:
         try:
             problems = linter.run(sys.stdin, conf, '')
-        except EnvironmentError as e:
+        except OSError as e:
             print(e, file=sys.stderr)
             sys.exit(-1)
         prob_level = show_problems(problems, 'stdin', args_format=args.format,
