@@ -14,6 +14,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import re
+import io
 
 import yaml
 
@@ -33,7 +34,7 @@ DISABLE_RULE_PATTERN = re.compile(r'^# yamllint disable( rule:\S+)*\s*$')
 ENABLE_RULE_PATTERN = re.compile(r'^# yamllint enable( rule:\S+)*\s*$')
 
 
-class LintProblem(object):
+class LintProblem:
     """Represents a linting problem found by yamllint."""
     def __init__(self, line, column, desc='<no description>', rule=None):
         #: Line on which the problem was found (starting at 1)
@@ -121,7 +122,7 @@ def get_cosmetic_problems(buffer, conf, filepath):
                         if id in self.all_rules:
                             self.rules.add(id)
 
-    # Use a cache to store problems and flush it only when a end of line is
+    # Use a cache to store problems and flush it only when an end of line is
     # found. This allows the use of yamllint directive to disable some rules on
     # some lines.
     cache = []
@@ -222,12 +223,12 @@ def run(input, conf, filepath=None):
     :param input: buffer, string or stream to read from
     :param conf: yamllint configuration object
     """
-    if conf.is_file_ignored(filepath):
+    if filepath is not None and conf.is_file_ignored(filepath):
         return ()
 
     if isinstance(input, (bytes, str)):
         return _run(input, conf, filepath)
-    elif hasattr(input, 'read'):  # Python 2's file or Python 3's io.IOBase
+    elif isinstance(input, io.IOBase):
         # We need to have everything in memory to parse correctly
         content = input.read()
         return _run(content, conf, filepath)
